/*****************************************************************************
 *
 * \file
 *
 * \brief CAN Software Stack Example 2.
 *
 * Copyright (c) 2006-2011 Atmel Corporation. All rights reserved.
 *
 * \asf_license_start
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The name of Atmel may not be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with an
 *    Atmel microcontroller product.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * \asf_license_stop
 *
 ******************************************************************************/

/*! \mainpage
 * \section intro Introduction
 * This documents data structures, functions, variables, defines, enums, and
 * typedefs in the software for the CAN driver.
 *
 * The given example is a CAN driver example using two CAN channels available.
 * A mailbox is defined by the user and all messages are sent easily.
 *
 * \section compinfo Compilation Info
 * This software was written for the GNU GCC for AVR32 and IAR Systems compiler
 * for AVR32. Other compilers may or may not work.
 *
 * \section files Main Files
 * - canif.c: CAN low level driver.
 * - canif.h: CAN low level header file.
 * - can.c: CAN High level driver.
 * - can.h: CAN High level header file.
 * - conf_can.h: Configuration file for the CAN Lib.
 * - can_example2.c: CAN Driver example.
 * - conf_can_example.h: Configuration file for the CAN example.
 *
 * \section configinfo Configuration Information
 * This example has been tested with the following configuration:
 * - UC3C_EK or UC3_C2_XPLAINED evaluation kit;
 * - CPU clock: 16 MHz;
 * This example demonstrates that how to send/receive messages from the CAN.
 * - Plug a DB9 cable on CAN1 DB9 connector and an external CAN node to setup a 'point to point' network.
 * - Configure the baudrate to 1Mbps.
 * - Start the example.
 * - A menu is displayed
 *         -- 1) If choose 1), the UC3C_EK or UC3_C2_XPLAINED  will send a message on CAN node. In case the message is correctly sent, a message is displayed.
 *         -- 2) If choose 2), the UC3C_EK or UC3_C2_XPLAINED will receive a message on CAN node. In case the message is correctly received, a message is displayed with the content of the FRAME.
 *         -- 3) If choose 3), the UC3C_EK or UC3_C2_XPLAINED will send a remote message on CAN node. In case the message is correctly sent, a message is displayed.
 * - The CPU is in sleep mode (STATIC) at the beginning of the application and then is waked up after the reception
 *   of the message from the external CAN node
 *
 *
 * \section contactinfo Contact Information
 * For further information, visit
 * <A href="http://www.atmel.com/products/AVR32/">Atmel AVR32</A>.\n
 */

#include <avr32/io.h>
#include "compiler.h"
#include "board.h"
#include "power_clocks_lib.h"
#include "gpio.h"
#include "pm_uc3c.h"
#include "scif_uc3c.h"
#include "intc.h"
#include "tc.h"
#include "can.h"
#include "canif.h"
#include "usart.h"
#include "print_funcs.h"
#include "conf_can_example.h"


//! Local allocation for MOB buffer 
volatile can_msg_t mob_ram_ch0[NB_MOB_CHANNEL];
volatile can_msg_t mob_ram_ch1[NB_MOB_CHANNEL];

volatile avr32_tc_t* tc = EXAMPLE_TC;

//! Local function to prepare RX and TX buffers
void can_kanal_0(uint8_t mode);
void can_kanal_1(uint8_t mode);

bool tx_msg3C1 = false;
bool rx_msg3C1 = false;
bool tx_msg3C2= false;
bool rx_msg3C2 = false;

//! Call Back prototype
void can_out_callback_channel0(U8 handle, U8 event);
void can_out_callback_channel1(U8 handle, U8 event);

int state0 = 0;
int state1 = 0;
int counter0 = 0;
int counter1 = 0;

//! The USART interrupt handler.
#if defined (__GNUC__)
__attribute__((__interrupt__))
#elif defined (__ICCAVR32__)
__interrupt
#endif

static void tc_irq(void)
{
	static int value = 1;
	// Clear the interrupt flag. This is a side effect of reading the TC SR.
	tc_read_sr(EXAMPLE_TC, EXAMPLE_TC_CHANNEL);
	
	switch(value)
	{
		case 1:
			tx_msg3C1 = true;
			value = 2;
			break;
			
		case 2:
			rx_msg3C1 = true;
			value = 3;
			break;
			
		case 3:
			tx_msg3C2 = true;
			value = 4;
			break;
			
		case 4:
			rx_msg3C2 = true;
			value = 1;
			break;
			
		default:
			break;
	}
	
}

/**
 * \brief TC Initialization
 *
 * Initializes and start the TC module with the following:
 * - Counter in Up mode with automatic reset on RC compare match.
 * - fPBA/8 is used as clock source for TC
 * - Enables RC compare match interrupt
 * \param tc Base address of the TC module
 */
static void tc_init(volatile avr32_tc_t *tc)
{
	// Options for waveform generation.
	static const tc_waveform_opt_t waveform_opt = {
		// Channel selection.
		.channel  = EXAMPLE_TC_CHANNEL,
		// Software trigger effect on TIOB.
		.bswtrg   = TC_EVT_EFFECT_NOOP,
		// External event effect on TIOB.
		.beevt    = TC_EVT_EFFECT_NOOP,
		// RC compare effect on TIOB.
		.bcpc     = TC_EVT_EFFECT_NOOP,
		// RB compare effect on TIOB.
		.bcpb     = TC_EVT_EFFECT_NOOP,
		// Software trigger effect on TIOA.
		.aswtrg   = TC_EVT_EFFECT_NOOP,
		// External event effect on TIOA.
		.aeevt    = TC_EVT_EFFECT_NOOP,
		// RC compare effect on TIOA.
		.acpc     = TC_EVT_EFFECT_NOOP,
		/*
		 * RA compare effect on TIOA.
		 * (other possibilities are none, set and clear).
		 */
		.acpa     = TC_EVT_EFFECT_NOOP,
		/*
		 * Waveform selection: Up mode with automatic trigger(reset)
		 * on RC compare.
		 */
		.wavsel   = TC_WAVEFORM_SEL_UP_MODE_RC_TRIGGER,
		// External event trigger enable.
		.enetrg   = false,
		// External event selection.
		.eevt     = 0,
		// External event edge selection.
		.eevtedg  = TC_SEL_NO_EDGE,
		// Counter disable when RC compare.
		.cpcdis   = false,
		// Counter clock stopped with RC compare.
		.cpcstop  = false,
		// Burst signal selection.
		.burst    = false,
		// Clock inversion.
		.clki     = false,
		// Internal source clock 3, connected to fPBA / 8.
		//.tcclks   = TC_CLOCK_SOURCE_TC3
		// Internal source clock 5 connected to fPBA / 128.
		.tcclks   = TC_CLOCK_SOURCE_TC4
	};

	// Options for enabling TC interrupts
	static const tc_interrupt_t tc_interrupt = {
		.etrgs = 0,
		.ldrbs = 0,
		.ldras = 0,
		.cpcs  = 1, // Enable interrupt on RC compare alone
		.cpbs  = 0,
		.cpas  = 0,
		.lovrs = 0,
		.covfs = 0
	};
	
	// Register the RTC interrupt handler to the interrupt controller.
	INTC_register_interrupt(&tc_irq, EXAMPLE_TC_IRQ, EXAMPLE_TC_IRQ_PRIORITY);
	
	// Initialize the timer/counter.
	tc_init_waveform(tc, &waveform_opt);

	/*
	 * Set the compare triggers.
	 * We configure it to get an interrupt every 0,5 seconds.
	 * We want: (1 / (fPBA / 8)) * RC = 1 ms, hence RC = (fPBA / 8) / 1000
	 */
	tc_write_rc(tc, EXAMPLE_TC_CHANNEL, ((FPBA_HZ / 32 /10) -1) );
	
	// configure the timer interrupt
	tc_configure_interrupts(tc, EXAMPLE_TC_CHANNEL, &tc_interrupt);
	
	// Start the timer/counter.
	tc_start(tc, EXAMPLE_TC_CHANNEL);
}


//! Call Back called for transmission
void can_out_callback_channel0(U8 handle, U8 event)
{
	  gpio_tgl_gpio_pin(LED0_GPIO);
	  switch (state0)
	  {
		  case 0: 
			can_mob_free(0,handle);
			break;
		
		  case 1: 
			receive_msg2[0].can_msg->data.u64 = can_get_mob_data(0,handle).u64;
			receive_msg2[0].can_msg->id = can_get_mob_id(0,handle);
			receive_msg2[0].dlc = can_get_mob_dlc(0,handle);
			receive_msg2[0].status = event;
			can_mob_free(0,handle);
			
			if( receive_msg2[0].can_msg->data.u64 == counter1 )
			{
				gpio_tgl_gpio_pin(LED2_GPIO);
			}
			
			break;
	  } 
}

void can_out_callback_channel1(U8 handle, U8 event)
{
	gpio_tgl_gpio_pin(LED1_GPIO);
		
	switch(state1)
	{
		case 0:
			can_mob_free(1,handle);
			break;
			
		case 1:
			receive_msg1[0].can_msg->data.u64 = can_get_mob_data(1,handle).u64;
			receive_msg1[0].can_msg->id = can_get_mob_id(1,handle);
			receive_msg1[0].dlc = can_get_mob_dlc(1,handle);
			receive_msg1[0].status = event;
			can_mob_free(1,handle);
			
			if( receive_msg1[0].can_msg->data.u64 == counter0 )
			{
				gpio_tgl_gpio_pin(LED3_GPIO);
			}
	}
	
	
}

/*! \brief Initializes the MCU system clocks.
 */
static void init_sys_clocks(void)
{
  // Switch the main clock to OSC0
  scif_configure_osc_crystalmode(SCIF_OSC0, FOSC0);
  scif_enable_osc(SCIF_OSC0, OSC0_STARTUP, true);
  pm_set_mclk_source(PM_CLK_SRC_OSC0);

  // Setup the generic clock for CAN
  scif_gc_setup(AVR32_SCIF_GCLK_CANIF, SCIF_GCCTRL_OSC0, AVR32_SCIF_GC_NO_DIV_CLOCK, 0);
  // Now enable the generic clock
  scif_gc_enable(AVR32_SCIF_GCLK_CANIF);
}

/*! \brief Prepare Data on CAN Bus.
 */
void can_kanal_0(uint8_t mode)
{
	 if(mode == 0)
	 {
		  // Initialize channel 0
		  can_init(0, ((U32)&mob_ram_ch0[0]), CANIF_CHANNEL_MODE_NORMAL, can_out_callback_channel0);
		  // Allocate one mob for TX
		  transmit_msg1[0].handle = can_mob_alloc(0);
	  
		  counter0++;
		  transmit_msg1[0].can_msg->data.u64 = counter0;
	  
		  can_tx(0, transmit_msg1[0].handle, transmit_msg1[0].dlc, transmit_msg1[0].req_type, transmit_msg1[0].can_msg);	
	 }
	 
	 if(mode == 1)
	 {
		 // Initialize channel 0
		 can_init(0, ((U32)&mob_ram_ch0[0]), CANIF_CHANNEL_MODE_NORMAL, can_out_callback_channel0);
		 // Allocate one mob for TX
		 receive_msg2[0].handle = can_mob_alloc(0);
		 
		 can_rx(0, receive_msg2[0].handle, receive_msg2[0].req_type, receive_msg2[0].can_msg);
	 }		 		 
     
}


void can_kanal_1(uint8_t mode)
{
	if(mode == 0)
	{
		// Initialize channel 0
		can_init(1, ((U32)&mob_ram_ch1[0]), CANIF_CHANNEL_MODE_NORMAL, can_out_callback_channel1);
		// Allocate one mob for TX
		transmit_msg2[0].handle = can_mob_alloc(1);
		
		counter1++;
		transmit_msg2[0].can_msg->data.u64 = counter1;
		
		can_tx(1, transmit_msg2[0].handle, transmit_msg2[0].dlc, transmit_msg2[0].req_type, transmit_msg2[0].can_msg);
	}
	
	if(mode == 1)
	{
		// Initialize channel 0
		can_init(1, ((U32)&mob_ram_ch1[0]), CANIF_CHANNEL_MODE_NORMAL, can_out_callback_channel1);
		// Allocate one mob for TX
		receive_msg1[0].handle = can_mob_alloc(1);
	
		can_rx(1, receive_msg1[0].handle, receive_msg1[0].req_type, receive_msg1[0].can_msg);
	}
	
}

int main(void)
{
	  init_sys_clocks();

	  init_dbg_rs232(FPBA_HZ);

	  // Disable all interrupts.
	  Disable_global_interrupt();

	  // Initialize interrupt vectors.
	  INTC_init_interrupts();

	  // Register the USART interrupt handler to the interrupt controller.
	  // usart_int_handler is the interrupt handler to register.
	  // EXAMPLE_USART_IRQ is the IRQ of the interrupt handler to register.
	  // AVR32_INTC_INT0 is the interrupt priority level to assign to the group of
	  // this IRQ.
	  // void INTC_register_interrupt(__int_handler handler, unsigned int irq, unsigned int int_level);
	  //INTC_register_interrupt(&usart_int_handler, UART_USART_IRQ, UART_USART_IRQ_LEVEL);

	  // Enable USART Rx interrupt.
	  UART_USART->ier = AVR32_USART_IER_RXRDY_MASK;
  
	  static const gpio_map_t CAN_GPIO_MAP = 
	  {
			{AVR32_CANIF_RXLINE_0_0_PIN, AVR32_CANIF_RXLINE_0_0_FUNCTION},
			{AVR32_CANIF_TXLINE_0_0_PIN, AVR32_CANIF_TXLINE_0_0_FUNCTION},
			{AVR32_CANIF_RXLINE_1_2_PIN, AVR32_CANIF_RXLINE_1_2_FUNCTION},
			{AVR32_CANIF_TXLINE_1_2_PIN, AVR32_CANIF_TXLINE_1_2_FUNCTION}
	  };
	  // Assign GPIO to CAN.
	  gpio_enable_module( CAN_GPIO_MAP, sizeof(CAN_GPIO_MAP) / sizeof(CAN_GPIO_MAP[0]) );
					 
	  tc_init(tc);	

	  // Enable all interrupts.
	  Enable_global_interrupt();

	  while(1)
	  {	  
		  if(tx_msg3C1)
		  {
			  state0 = 0;
			  can_kanal_0(0);
			  tx_msg3C1 = false;
		  } 
		  if (rx_msg3C1)
		  {
			  state1 = 1; 
			  can_kanal_1(1);
			  rx_msg3C1 = false;
		  } 
		  if (tx_msg3C2)
		  {
			  state1 = 0; 
			  can_kanal_1(0);
			  tx_msg3C2 = false;
		  }
		  if (rx_msg3C2)
		  {
			  state0 = 1;
			  can_kanal_0(1);
			  rx_msg3C2 = false;
		  }
		
		  
	  }       
}
